<?php
namespace Internetgalerie\IgsCrm\Domain\Model;

use Internetgalerie\IgsCrm\Domain\Model\Contact;
use Internetgalerie\IgsCrm\Domain\Repository\ContactRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Domain\Model\FileReference;
use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
    
// @todo \TYPO3\CMS\Extbase\Domain\Model\FrontendUser will removed in v12

class FrontendUser extends AbstractEntity
{
    /**
     * @var string
     */
    protected $username = '';

    /**
     * @var string
     */
    protected $password = '';

    /**
     * @var ObjectStorage<FrontendUserGroup>
     */
    protected $usergroup;

    /**
     * @var string
     */
    protected $name = '';

    /**
     * @var string
     */
    protected $firstName = '';

    /**
     * @var string
     */
    protected $middleName = '';

    /**
     * @var string
     */
    protected $lastName = '';

    /**
     * @var string
     */
    protected $address = '';

    /**
     * @var string
     */
    protected $telephone = '';

    /**
     * @var string
     */
    protected $fax = '';

    /**
     * @var string
     */
    protected $email = '';

    /**
     * @var string
     */
    protected $title = '';

    /**
     * @var string
     */
    protected $zip = '';

    /**
     * @var string
     */
    protected $city = '';

    /**
     * @var ObjectStorage<FileReference>
     */
    protected $image;

    
    /**
     * crmSettingsPrinterMarginTop
     *
     * @var int
     */
    protected $crmSettingsPrinterMarginTop;

    /**
     * crmSettingsCustomCss
     *
     * @var string
     */
    protected $crmSettingsCustomCss;

    /**
     * crmVerbandUids
     *
     * @var string
     */
    protected $crmVerbandUids;

    /**
     * crmDefaultGroup
     *
     * @var int
     */
    protected $crmDefaultGroup;


    /**
     * Constructs a new Front-End User
     */
    public function __construct()
    {
        $this->usergroup = new ObjectStorage();
        $this->image = new ObjectStorage();
    }

    /**
     * Called again with initialize object, as fetching an entity from the DB does not use the constructor
     */
    public function initializeObject()
    {
        $this->usergroup = $this->usergroup ?? new ObjectStorage();
        $this->image = $this->image ?? new ObjectStorage();
    }


    /**
     * Sets the username value
     *
     * @param string $username
     */
    public function setUsername($username)
    {
        $this->username = $username;
    }

    /**
     * Returns the username value
     *
     * @return string
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * Sets the password value
     *
     * @param string $password
     */
    public function setPassword($password)
    {
        $this->password = $password;
    }

    /**
     * Returns the password value
     *
     * @return string
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Sets the usergroups. Keep in mind that the property is called "usergroup"
     * although it can hold several usergroups.
     *
     * @param ObjectStorage<FrontendUserGroup> $usergroup
     */
    public function setUsergroup(ObjectStorage $usergroup)
    {
        $this->usergroup = $usergroup;
    }

    /**
     * Adds a usergroup to the frontend user
     *
     * @param FrontendUserGroup $usergroup
     */
    public function addUsergroup(FrontendUserGroup $usergroup)
    {
        $this->usergroup->attach($usergroup);
    }

    /**
     * Removes a usergroup from the frontend user
     *
     * @param FrontendUserGroup $usergroup
     */
    public function removeUsergroup(FrontendUserGroup $usergroup)
    {
        $this->usergroup->detach($usergroup);
    }

    /**
     * Returns the usergroups. Keep in mind that the property is called "usergroup"
     * although it can hold several usergroups.
     *
     * @return ObjectStorage<FrontendUserGroup> An object storage containing the usergroup
     */
    public function getUsergroup()
    {
        return $this->usergroup;
    }

    /**
     * Sets the name value
     *
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Returns the name value
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Sets the firstName value
     *
     * @param string $firstName
     */
    public function setFirstName($firstName)
    {
        $this->firstName = $firstName;
    }

    /**
     * Returns the firstName value
     *
     * @return string
     */
    public function getFirstName()
    {
        return $this->firstName;
    }

    /**
     * Sets the middleName value
     *
     * @param string $middleName
     */
    public function setMiddleName($middleName)
    {
        $this->middleName = $middleName;
    }

    /**
     * Returns the middleName value
     *
     * @return string
     */
    public function getMiddleName()
    {
        return $this->middleName;
    }

    /**
     * Sets the lastName value
     *
     * @param string $lastName
     */
    public function setLastName($lastName)
    {
        $this->lastName = $lastName;
    }

    /**
     * Returns the lastName value
     *
     * @return string
     */
    public function getLastName()
    {
        return $this->lastName;
    }

    /**
     * Sets the address value
     *
     * @param string $address
     */
    public function setAddress($address)
    {
        $this->address = $address;
    }

    /**
     * Returns the address value
     *
     * @return string
     */
    public function getAddress()
    {
        return $this->address;
    }

    /**
     * Sets the telephone value
     *
     * @param string $telephone
     */
    public function setTelephone($telephone)
    {
        $this->telephone = $telephone;
    }

    /**
     * Returns the telephone value
     *
     * @return string
     */
    public function getTelephone()
    {
        return $this->telephone;
    }

    /**
     * Sets the fax value
     *
     * @param string $fax
     */
    public function setFax($fax)
    {
        $this->fax = $fax;
    }

    /**
     * Returns the fax value
     *
     * @return string
     */
    public function getFax()
    {
        return $this->fax;
    }

    /**
     * Sets the email value
     *
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * Returns the email value
     *
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Sets the title value
     *
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * Returns the title value
     *
     * @return string
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * Sets the zip value
     *
     * @param string $zip
     */
    public function setZip($zip)
    {
        $this->zip = $zip;
    }

    /**
     * Returns the zip value
     *
     * @return string
     */
    public function getZip()
    {
        return $this->zip;
    }

    /**
     * Sets the city value
     *
     * @param string $city
     */
    public function setCity($city)
    {
        $this->city = $city;
    }

    /**
     * Returns the city value
     *
     * @return string
     */
    public function getCity()
    {
        return $this->city;
    }
    

    /**
     * Sets the image value
     *
     * @param ObjectStorage<FileReference> $image
     */
    public function setImage(ObjectStorage $image)
    {
        $this->image = $image;
    }

    /**
     * Gets the image value
     *
     * @return ObjectStorage<FileReference>
     */
    public function getImage()
    {
        return $this->image;
    }

    
    /**
     * Returns the crmSettingsPrinterMarginTop
     *
     * @return int $crmSettingsPrinterMarginTop
     */
    public function getCrmSettingsPrinterMarginTop()
    {
        return $this->crmSettingsPrinterMarginTop;
    }

    /**
     * Sets the crmSettingsPrinterMarginTop
     *
     * @param int $crmSettingsPrinterMarginTop
     * @return void
     */
    public function setCrmSettingsPrinterMarginTop($crmSettingsPrinterMarginTop)
    {
        $this->crmSettingsPrinterMarginTop = $crmSettingsPrinterMarginTop;
    }

    /**
     * Returns the crmSettingsCustomCss
     *
     * @return string $crmSettingsCustomCss
     */
    public function getCrmSettingsCustomCss()
    {
        return $this->crmSettingsCustomCss;
    }

    /**
     * Sets the crmSettingsCustomCss
     *
     * @param string $crmSettingsCustomCss
     * @return void
     */
    public function setCrmSettingsCustomCss($crmSettingsCustomCss)
    {
        $this->crmSettingsCustomCss = $crmSettingsCustomCss;
    }

    /**
     * Returns the crmVerbandUids
     *
     * @return string $crmVerbandUids
     */
    public function getCrmVerbandUids()
    {
        return $this->crmVerbandUids;
    }

    /**
     * Sets the crmVerbandUids
     *
     * @param string $crmVerbandUids
     * @return void
     */
    public function setCrmVerbandUids($crmVerbandUids)
    {
        $this->crmVerbandUids = $crmVerbandUids;
    }


    /**
     * Returns the crmDefaultGroup
     *
     * @return string $crmDefaultGroup
     */
    public function getCrmDefaultGroup()
    {
        return $this->crmDefaultGroup;
    }

    /**
     * Sets the crmDefaultGroup
     *
     * @param string $crmDefaultGroup
     * @return void
     */
    public function setCrmDefaultGroup($crmDefaultGroup)
    {
        $this->crmDefaultGroup = $crmDefaultGroup;
    }
    

    /**
     * Returns the contact of this frontendUser
     */
    public function getContact(): ?Contact
    {
        $contacts = $this->getContacts();
        return empty($contacts) ? null : $contacts[0];
    }


    /**
     * Returns the contact of this frontendUser
     *
     * @return array
     */
    public function getContacts()
    {
        $this->contactRepository =  GeneralUtility::makeInstance(ContactRepository::class);
        return $this->contactRepository->findByFrontendUser($this->uid);
    }


}
