<?php
namespace Internetgalerie\IgsCrm\Domain\Model;


/**
 * Mitgliedschaft
 */

class Mitgliedschaft extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * name
     *
     * @var string
     */
    protected $name = '';

    /**
     * code
     *
     * @var string
     */
    protected $code = '';

    /**
     * limit_to_type
     *
     * @var string
     */
    protected $limitToType = '';

    /**
     * mgBetrag
     *
     * @var string
     */
    protected $betrag = '';

    /**
     * betragIsTotal
     *
     * @var boolean
     */
    protected $betragIsTotal = false;


    /**
     * invoicestext
     *
     * @var string
     */
    protected $invoicestext = '';

    /**
     * noPerson
     *
     * @var boolean
     */
    protected $noPerson = false;


    /**
     * verband
     *
     * @var \Internetgalerie\IgsCrm\Domain\Model\Verband
     */
    public $verband = null;


    /**
     * card
     *
     * @var \Internetgalerie\IgsCrm\Domain\Model\Card
     */
    public $card = null;

    /**
     * preise
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Preise>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    public $preise = null;

    /**
     * __construct
     */
    public function __construct()
    {
        //Do not remove the next line: It would break the functionality
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    protected function initStorageObjects()
    {
        $this->preise = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Returns the name
     *
     * @return string $name
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Sets the name
     *
     * @param string $name
     * @return void
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Returns the code
     *
     * @return string $code
     */
    public function getCode()
    {
        return $this->code;
    }

    /**
     * Sets the code
     *
     * @param string $code
     * @return void
     */
    public function setCode($code)
    {
        $this->code = $code;
    }

    /**
     * Returns the limitToType
     *
     * @return string $limitToType
     */
    public function getLimitToType()
    {
        return $this->limitToType;
    }

    /**
     * Sets the limitToType
     *
     * @param string $limitToType
     * @return void
     */
    public function setLimitToType($limitToType)
    {
        $this->limitToType = $limitToType;
    }

    /**
     * Returns the betrag
     *
     * @return string $betrag
     */
    public function getBetrag()
    {
        return $this->betrag;
    }
    /**
     * get amount for contact for given year (amount is depending of age of contact in given year)
     * 
     * @param Contact $contact Contact for age and gender
     * @param int $year year to calculate age
     * @return amount 
     */    
    public function getAmountForContact(Contact $contact, int $year = null)
    {
        $addYear = $year === null ? 0 : $year - date('Y');
        $alterWithYear = $contact->getMeAlterWithYear() + $addYear;
        return $this->getBetragWithAlter($alterWithYear, $contact->getAnredeId());
    }

    public function getBetragWithAlter($alterInJahre, $anrede)
    {
        if (intval($alterInJahre) <= 0) {
            return $this->betrag;
        }
        $ret = $this->betrag;
        if (!empty($this->getPreise())) {
            foreach ($this->getPreise() as $preis) {
                //echo('test: ' . $preis->getAlterInJahrVon() .'==='. $alterInJahre .' , '.  $preis->getAnrede() . '==='.$anrede .'<br />');
                if ($preis->getAlterInJahrVon() <= $alterInJahre && $preis->getAlterInJahrBis() >= $alterInJahre) {
                    if ($preis->getAnrede() == $anrede) {
                        return $preis->getBetrag();
                    } elseif ($preis->getAnrede() == 0) {
                        $ret = $preis->getBetrag();
                    }
                }
            }
        }
        return $ret;
    }

    /**
     * Sets the betrag
     *
     * @param string $betrag
     * @return void
     */
    public function setBetrag($betrag)
    {
        $this->betrag = $betrag;
    }

    /**
     * Returns the betragIsTotal
     *
     * @return boolean $betragIsTotal
     */
    public function getBetragIsTotal()
    {
        return $this->betragIsTotal;
    }

    /**
     * Sets the betragIsTotal
     *
     * @param boolean $betragIsTotal
     * @return void
     */
    public function setBetragIsTotal($betragIsTotal)
    {
        $this->betragIsTotal = $betragIsTotal;
    }



    /**
     * Returns the noPerson
     *
     * @return boolean $noPerson
     */
    public function getNoPerson()
    {
        return $this->noPerson;
    }

    /**
     * Sets the noPerson
     *
     * @param boolean $noPerson
     * @return void
     */
    public function setNoPerson($noPerson)
    {
        $this->noPerson = $noPerson;
    }



    /**
     * Returns the invoicestext
     *
     * @return string $invoicestext
     */
    public function getInvoicestext()
    {
        return $this->invoicestext;
    }
    public function getInvoicestextWithAlter($alterInJahre)
    {
        if (intval($alterInJahre)<=0) {
            return $this->invoicestext;
        }
        if (!empty($this->getPreise())) {
            foreach ($this->getPreise() as $preis) {
                if ($preis->getAlterInJahrVon()<=$alterInJahre && $preis->getAlterInJahrBis()>=$alterInJahre) {
                    return $preis->getInvoicestext() ?? $this->invoicestext;
                }
            }
        }
        return $this->invoicestext;
    }

    /**
     * Sets the invoicestext
     *
     * @param string $invoicestext
     * @return void
     */
    public function setInvoicestext($invoicestext)
    {
        $this->invoicestext = $invoicestext;
    }


    /**
     * Returns the verband
     *
     * @return \Internetgalerie\IgsCrm\Domain\Model\Verband $verband
     */
    public function getVerband()
    {
        return $this->verband;
    }

    /**
     * Sets the verband
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Verband $verband
     * @return void
     */
    public function setVerband(\Internetgalerie\IgsCrm\Domain\Model\Verband $verband)
    {
        $this->verband = $verband;
    }
    /**
     * Adds a Preise
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Preise $preise
     * @return void
     */
    public function addPreise(\Internetgalerie\IgsCrm\Domain\Model\Preise $preise)
    {
        $this->preise->attach($preise);
    }

    /**
     * Removes a Preise
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Preise $preiseToRemove The Preise to be removed
     * @return void
     */
    public function removePreise(\Internetgalerie\IgsCrm\Domain\Model\Preise $preiseToRemove)
    {
        $this->preise->detach($preiseToRemove);
    }

    /**
     * Returns the preise
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Preise> $preise
     */
    public function getPreise()
    {
        return $this->preise;
    }

    /**
     * Sets the preise
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Preise> $preise
     * @return void
     */
    public function setPreise(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $preise)
    {
        $this->preise = $preise;
    }

    /**
     * Get the value of card
     *
     * @return \Internetgalerie\IgsCrm\Domain\Model\Card
     */
    public function getCard()
    {
        return $this->card;
    }

    /**
     * Set the value of card
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Card $card
     *
     * @return self
     */
    public function setCard(\Internetgalerie\IgsCrm\Domain\Model\Card $card = null)
    {
        $this->card = $card;

        return $this;
    }
    public function __toString()
    {
        return $this->getName();
    }
}
