<?php
namespace Internetgalerie\IgsCrm\Domain\Model;

use Internetgalerie\IgCrmTemplate\Domain\Model\AddressInterface;
use Internetgalerie\IgCrmTemplate\Domain\Model\TenantInterface;
use Ig\IgFibu\Domain\Model\ClientInterface;
use Ig\IgFibu\ReferenceFormatter\DefaultReferenceFormatter;
use Internetgalerie\IgCrmTemplate\Domain\Repository\TemplateLetterRepository;
use Internetgalerie\IgCrmTemplate\Utility\CrmTranslate;
use Internetgalerie\IgCrmTemplate\Domain\Repository\TranslateRepository;
use Internetgalerie\IgsCrm\Domain\Model\Contact;
use Internetgalerie\IgsCrm\Domain\Model\EntityWithAcl;
use Internetgalerie\IgsCrm\Domain\Model\FrontendUserGroup;
use Internetgalerie\IgsCrm\Domain\Model\Language;
use Internetgalerie\IgsCrm\Domain\Model\Invoice;
use Internetgalerie\IgsCrm\Domain\Repository\ZeitschriftRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Annotation as Extbase;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Extbase\Domain\Model\FileReference;

/**
 * Verband
 */
class Verband extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity implements AddressInterface, EntityWithAcl, ClientInterface, TenantInterface
{

    /**
     * parent
     *
     * @var Verband
     */
    protected $parent = null;

    /**
     * sorting
     *
     * @var integer
     */
    protected $sorting = 0;

    /**
     * pathUids
     *
     * @var string
     */
    protected $pathUids = '';

    /**
     * name
     *
     * @var string
     */
    protected $name = '';

    /**
     * nameHr
     *
     * @var string
     */
    protected $nameHr = '';

    /**
     * nameHr
     *
     * @var string
     */
    protected $nameFibu = '';

    /**
     * nameDe
     *
     * @var string
     */
    protected $nameDe = '';

    /**
     * nameFr
     *
     * @var string
     */
    protected $nameFr = '';

    /**
     * nameIt
     *
     * @var string
     */
    protected $nameIt = '';

    /**
     * nameEn
     *
     * @var string
     */
    protected $nameEn = '';

    /**
     * addon
     *
     * @var string
     */
    protected $addon = '';

    /**
     * bfsUid
     *
     * @var string
     */
    protected $bfsUid = '';

    /**
     * email
     *
     * @var string
     */
    protected $email = '';

    /**
     * url
     *
     * @var string
     */
    protected $url = '';

    /**
     * bankingCustomerIdentification
     *
     * @var string
     */
    protected $bankingCustomerIdentification = '';

    /**
     * iban
     *
     * @var string
     */
    protected $iban = '';

    /**
     * qrIban
     *
     * @var string
     */
    protected $qrIban = '';

    /**
     * bankName
     *
     * @var string
     */
    protected $bankName = '';

    /**
     * bankZipCity
     *
     * @var string
     */
    protected $bankZipCity = '';

    /**
     * bankKonto
     *
     * @var string
     */
    protected $bankKonto = '';

    /**
     * mwstNr
     *
     * @var string
     */
    protected $mwstNr = '';

    /**
     * invoiceFromEmail
     *
     * @var string
     */
    protected $invoiceFromEmail = '';

    /**
     * invoiceFromName
     *
     * @var string
     */
    protected $invoiceFromName = '';

    /**
     * invoiceTestEmail
     *
     * @var string
     */
    protected $invoiceTestEmail = '';

    /**
     * invoiceBccEmail
     *
     * @var string
     */
    protected $invoiceBccEmail = '';

   /**
     * invoicePeriodeLengthInMonth
     *
     * @var int
     */
    protected $invoicePeriodeLengthInMonth;

    /**
     * invoiceText
     *
     * @var string
     */
    protected $invoiceText = '';

    /**
     * useOwnAddressNumber
     *
     * @var bool
     */
    protected $useOwnAddressNumber = false;

    /**
     * invoiceCss
     *
     * @var string
     */
    protected $invoiceCss = '';

    /**
     * taxBehavior (0 = inkl., 1 = exkl)
     *
     * @var int
     */
    protected $taxBehavior = null;

    /**
     * taxRate
     *
     * @var float
     */
    protected $taxRate = null;

   /**
     * usePp
     *
     * @var bool
     */
    protected $usePp = false;

    /**
     * exportConfig
     *
     * @var string
     */
    protected $exportConfig = '';

    /**
     * logo
     */
    protected ?FileReference $logo = null;

    /**
     * logoWidth
     *
     * @var int
     */
    protected $logoWidth;

    /**
     * logoMail
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $logoMail = null;


    /**
     * logoMailWidth
     *
     * @var int
     */
    protected $logoMailWidth;

    /**
     * street
     *
     * @var string
     */
    protected $street = '';

    /**
     * streetNumber
     *
     * @var string
     */
    protected $streetNumber = '';

    /**
     * address2
     *
     * @var string
     */
    protected $address2 = '';

    /**
     * address3
     *
     * @var string
     */
    protected $address3 = '';

    /**
     * zip
     *
     * @var string
     */
    protected $zip = '';

    /**
     * countryIsoCode
     *
     * @var string
     */
    protected $countryIsoCode = '';

    /**
     * city
     *
     * @var string
     */
    protected $city = '';

    /**
     * cityDe
     *
     * @var string
     */
    protected $cityDe = '';

    /**
     * cityFr
     *
     * @var string
     */
    protected $cityFr = '';

    /**
     * cityIt
     *
     * @var string
     */
    protected $cityIt = '';

    /**
     * cityEn
     *
     * @var string
     */
    protected $cityEn = '';

    /**
     * phone
     *
     * @var string
     */
    protected $phone = '';


    /**
     * aclOwner
     *
     * @var int
     */
    public $aclOwner = 0;

    /**
     * aclWriteGroups
     *
     * @var ObjectStorage<FrontendUserGroup>
     */
    protected $aclWriteGroups;

    /**
     * aclReadGroups
     *
     * @var ObjectStorage<FrontendUserGroup>
     */
    protected $aclReadGroups;

    /**
     * languages
     *
     * @var ObjectStorage<Language>
     */
    protected $languages;

    /**
     * editJs
     *
     * @var string
     */
    protected $editJs = '';

    /**
     * editCss
     *
     * @var string
     */
    protected $editCss = '';

    /**
     * templateName
     *
     * @var string
     */
    protected $templateName = '';

    /**
     * Returns the name
     *
     * @return string $name
     */
    /**
     * titleExtraTextfield1
     *
     * @var string
     */
    protected $titleExtraTextfield1 = '';

    /**
     * titleExtraTextfield2
     *
     * @var string
     */
    protected $titleExtraTextfield2 = '';

    /**
     * titleExtraTextfield3
     *
     * @var string
     */
    protected $titleExtraTextfield3 = '';

    /**
     * titleExtraTextarea1
     *
     * @var string
     */
    protected $titleExtraTextarea1 = '';

    /**
     * titleExtraTextarea2
     *
     * @var string
     */
    protected $titleExtraTextarea2 = '';

    /**
     * titleExtraTextarea3
     *
     * @var string
     */
    protected $titleExtraTextarea3 = '';

    /**
     * titleExternalId
     *
     * @var string
     */
    protected $titleExternalId = '';

    /**
     * titleTags
     *
     * @var string
     */
    protected $titleTags = '';

    /**
     * titleCommissions
     *
     * @var string
     */
    protected $titleCommissions = '';

    /**
     * titleCategories
     *
     * @var string
     */
    protected $titleCategories = '';

    /**
     * veteranenAlter
     *
     * @var int
     */
    protected $veteranenAlter = 0;

    /**
     * alterskategorieMode
     *
     * @var int
     */
    protected $alterskategorieMode = 0;

    /**
     * journalMode
     *
     * @var int
     */
    protected $journalMode = 0;

    /**
     * invoiceMode
     *
     * @var int
     */
    protected $invoiceMode = 0;

    /**
     * personUidAttribute
     *
     * @var string
     */
    protected $personUidAttribute = '';

    /**
     * titleOrganisationId
     *
     * @var string
     */
    protected $titleOrganisationId = '';

    /**
     * titleJournalIsFree1
     *
     * @var string
     */
    protected $titleJournalIsFree1 = '';

    /**
     * titleJournalIsFree2
     *
     * @var string
     */
    protected $titleJournalIsFree2 = '';

    /**
     * invoiceBaseText
     *
     * @var string
     */
    protected $invoiceBaseText = '';

    /**
     * extraBetragTitle
     *
     * @var string
     */
    protected $extraBetragTitle = '';

    /**
     * extraBetragValue
     *
     * @var float
     */
    protected $extraBetragValue = 0.0;

    /**
     * testDates
     *
     * @var boolean
     */
    protected $testDates = false;

    /**
     * showInvoiceAddress
     *
     * @var boolean
     */
    protected $showInvoiceAddress = true;

    /**
     * exports
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Export>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    public $exports = null;

    /**
     * senderLine
     *
     * @var string
     */
    protected $senderLine = '';


    /**
     * categoryRepository
     *
     * @var \Internetgalerie\IgsCrm\Domain\Repository\CategoryRepository
     */
    protected $categoryRepository = null;

    /**
     * tagRepository
     *
     * @var \Internetgalerie\IgsCrm\Domain\Repository\TagRepository
     */
    protected $tagRepository = null;

    /**
     * tagverbandRepository
     *
     * @var \Internetgalerie\IgsCrm\Domain\Repository\TagverbandRepository
     */
    protected $tagverbandRepository = null;

    /**
     * mitgliedschaftRepository
     *
     * @var \Internetgalerie\IgsCrm\Domain\Repository\MitgliedschaftRepository
     */
    protected $mitgliedschaftRepository = null;

    /**
     * mitgliedschaftenExclude
     * @var QueryResultInterface
     */
    protected $mitgliedschaftenExclude = null;

    /**
     * templateLetters
     * @var QueryResultInterface
     */
    protected $templateLetters = null;

    /**
     * templateLetter
     * @var array
     */
    protected $templateLetter = [];

    /**
     * translates
     * @var QueryResultInterface
     */
    protected $translates = null;

    protected ?QueryResultInterface $cards = null;
    protected ?ZeitschriftRepository $zeitschriftRepository = null;

    /**
     * __construct
     */
    public function __construct()
    {
        $this->aclReadGroups = new ObjectStorage();
        $this->aclWriteGroups = new ObjectStorage();
        $this->languages = new ObjectStorage();
        $this->exports = new ObjectStorage();
        //Do not remove the next line: It would break the functionality
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties
     *
     * @return void
     */
    protected function initStorageObjects()
    {
        $this->exports = $this->exports ?? new ObjectStorage();
        $this->aclReadGroups = $this->aclReadGroups ?? new ObjectStorage();
        $this->aclWriteGroups = $this->aclWriteGroups ?? new ObjectStorage();
        $this->languages = $this->languages ?? new ObjectStorage();
    }
    public function getType()
    {
        return 'Verband';
    }

    public function getNumber()
    {
        return $this->uid;
    }

    /**
     * Returns the parent
     *
     * @return Verband $parent
     */
    public function getParent()
    {
        return $this->parent;
    }

    /**
     * Sets the parent
     *
     * @param Verband $parent
     * @return void
     */
    public function setParent(Verband $parent = null)
    {
        $this->parent = $parent;
    }

    /**
     * Returns the sorting
     *
     * @return integer $sorting
     */
    public function getSorting() {
        return $this->sorting;
    }

    /**
     * Sets the sorting
     *
     * @param integer $sorting
     * @return void
     */
    public function setSorting($sorting) {
        $this->sorting = $sorting;
    }

    /**
     * Returns the pathUids
     *
     * @return string $pathUids
     */
    public function getPathUids()
    {
        return $this->pathUids;
    }

    /**
     * Sets the pathUids
     *
     * @param string $pathUids
     * @return void
     */
    public function setPathUids($pathUids)
    {
        $this->pathUids = $pathUids;
    }

    public function getBreadcrumb()
    {
        $breadcrumb = [];
        $pathVerband = $this;
        $limit = 10;
        do {
            $limit--;
            $breadcrumb[] = $pathVerband;
            $pathVerband = $pathVerband->getParent();
        } while ($pathVerband && $limit > 0);
        return array_reverse($breadcrumb);
    }
    /**
     * Returns the name
     *
     * @return string $name
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Sets the name
     *
     * @param string $name
     * @return void
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Returns the nameHr
     *
     * @return string $nameHr
     */
    public function getNameHr()
    {
        return $this->nameHr;
    }


    /**
     * Sets the nameHr
     *
     * @param string $nameHr
     * @return void
     */
    public function setNameHr($nameHr)
    {
        $this->nameHr = $nameHr;
    }

    /**
     * Returns the nameFibu
     *
     * @return string $nameFibu
     */
    public function getNameFibu()
    {
        return $this->nameFibu;
    }

    /**
     * Sets the nameFibu
     *
     * @param string $nameFibu
     * @return void
     */
    public function setNameFibu($nameFibu)
    {
        $this->nameFibu = $nameFibu;
    }


    public function getOrganisationName()
    {
        return $this->nameHr;
    }
    public function getOrganisationNameFibu()
    {
        return $this->nameFibu ? : $this->nameHr;
    }


    /**
     * Returns the nameDe
     *
     * @return string $nameDe
     */
    public function getNameDe()
    {
        return $this->nameDe;
    }

    /**
     * Sets the nameDe
     *
     * @param string $nameDe
     * @return void
     */
    public function setNameDe($nameDe)
    {
        $this->nameDe = $nameDe;
    }
    /**
     * Returns the nameFr
     *
     * @return string $nameFr
     */
    public function getNameFr()
    {
        return $this->nameFr;
    }

    /**
     * Sets the nameFr
     *
     * @param string $nameFr
     * @return void
     */
    public function setNameFr($nameFr)
    {
        $this->nameFr = $nameFr;
    }

    /**
     * Returns the nameIt
     *
     * @return string $nameIt
     */
    public function getNameIt()
    {
        return $this->nameIt;
    }

    /**
     * Sets the nameIt
     *
     * @param string $nameIt
     * @return void
     */
    public function setNameIt($nameIt)
    {
        $this->nameIt = $nameIt;
    }

    /**
     * Returns the nameEn
     *
     * @return string $nameEn
     */
    public function getNameEn()
    {
        return $this->nameEn;
    }

    /**
     * Sets the nameEn
     *
     * @param string $nameEn
     * @return void
     */
    public function setNameEn($nameEn)
    {
        $this->nameEn = $nameEn;
    }
    public function getNameLocale()
    {
        switch ($GLOBALS['TSFE']->sys_language_uid) {
       case 1:
           return $this->nameEn ? : $this->nameHr;
       case 2:
           return $this->nameFr ? : $this->nameHr;
       case 3:
           return $this->nameIt ? : $this->nameHr;
       default:
           return $this->nameDe ? : $this->nameHr;
       }
    }

    /**
     * Returns the bfsUid
     *
     * @return string $bfsUid
     */
    public function getBfsUid()
    {
        return $this->bfsUid;
    }

    /**
     * Sets the bfsUid
     *
     * @param string $bfsUid
     * @return void
     */
    public function setBfsUid($bfsUid)
    {
        $this->bfsUid = $bfsUid;
    }

    /**
     * Returns the email
     *
     * @return string $email
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Sets the email
     *
     * @param string $email
     * @return void
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * E-Mail for FIBU adddress interface
     */
    public function getInvoiceEmail()
    {
        return $this->email;
    }

    /**
     * Returns the url
     *
     * @return string $url
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * Sets the url
     *
     * @param string $url
     * @return void
     */
    public function setUrl($url)
    {
        $this->url = $url;
    }

    /**
     * Returns the bankingCustomerIdentification
     *
     * @return string $bankingCustomerIdentification
     */
    public function getBankingCustomerIdentification()
    {
        return $this->bankingCustomerIdentification;
    }

    /**
     * Sets the bankingCustomerIdentification
     *
     * @param string $bankingCustomerIdentification
     * @return void
     */
    public function setBankingCustomerIdentification($bankingCustomerIdentification)
    {
        $this->bankingCustomerIdentification = $bankingCustomerIdentification;
    }

    /**
     * Returns the iban
     *
     * @return string $iban
     */
    public function getIban()
    {
        return $this->iban;
    }

    /**
     * Sets the iban
     *
     * @param string $iban
     * @return void
     */
    public function setIban($iban)
    {
        $this->iban = $iban;
    }

    /**
     * Returns the qrIban
     *
     * @return string $qrIban
     */
    public function getQrIban()
    {
        return $this->qrIban;
    }

    /**
     * Sets the qrIban
     *
     * @param string $qrIban
     * @return void
     */
    public function setQrIban($qrIban)
    {
        $this->qrIban = $qrIban;
    }

    /**
     * Returns the bankName
     *
     * @return string $bankName
     */
    public function getBankName()
    {
        return $this->bankName;
    }

    /**
     * Sets the bankName
     *
     * @param string $bankName
     * @return void
     */
    public function setBankName($bankName)
    {
        $this->bankName = $bankName;
    }

    /**
     * Returns the bankZipCity
     *
     * @return string $bankZipCity
     */
    public function getBankZipCity()
    {
        return $this->bankZipCity;
    }

    /**
     * Sets the bankZipCity
     *
     * @param string $bankZipCity
     * @return void
     */
    public function setBankZipCity($bankZipCity)
    {
        $this->bankZipCity = $bankZipCity;
    }

    /**
     * Returns the bankKonto
     *
     * @return string $bankKonto
     */
    public function getBankKonto()
    {
        return $this->bankKonto;
    }

    /**
     * Sets the bankKonto
     *
     * @param string $bankKonto
     * @return void
     */
    public function setBankKonto($bankKonto)
    {
        $this->bankKonto = $bankKonto;
    }


    /**
     * Returns the mwstNr
     *
     * @return string $mwstNr
     */
    public function getMwstNr()
    {
        return $this->mwstNr;
    }

    /**
     * Sets the mwstNr
     *
     * @param string $mwstNr
     * @return void
     */
    public function setMwstNr($mwstNr)
    {
        $this->mwstNr = $mwstNr;
    }


    /**
     * Returns the invoiceFromName
     *
     * @return string $invoiceFromName
     */
    public function getInvoiceFromName()
    {
        return $this->invoiceFromName;
    }

    /**
     * Sets the invoiceFromName
     *
     * @param string $invoiceFromName
     * @return void
     */
    public function setInvoiceFromName($invoiceFromName)
    {
        $this->invoiceFromName = $invoiceFromName;
    }

    /**
     * Returns the invoiceFromEmail
     *
     * @return string $invoiceFromEmail
     */
    public function getInvoiceFromEmail()
    {
        return $this->invoiceFromEmail;
    }

    /**
     * Sets the invoiceFromEmail
     *
     * @param string $invoiceFromEmail
     * @return void
     */
    public function setInvoiceFromEmail($invoiceFromEmail)
    {
        $this->invoiceFromEmail = $invoiceFromEmail;
    }

    /**
     * Returns the invoiceTestEmail
     *
     * @return string $invoiceTestEmail
     */
    public function getInvoiceTestEmail()
    {
        return $this->invoiceTestEmail;
    }

    /**
     * Sets the invoiceTestEmail
     *
     * @param string $invoiceTestEmail
     * @return void
     */
    public function setInvoiceTestEmail($invoiceTestEmail)
    {
        $this->invoiceTestEmail = $invoiceTestEmail;
    }

    /**
     * Returns the invoiceBccEmail
     *
     * @return string $invoiceBccEmail
     */
    public function getInvoiceBccEmail()
    {
        return $this->invoiceBccEmail;
    }

    /**
     * Sets the invoiceBccEmail
     *
     * @param string $invoiceBccEmail
     * @return void
     */
    public function setInvoiceBccEmail($invoiceBccEmail)
    {
        $this->invoiceBccEmail = $invoiceBccEmail;
    }
    
    /**
     * Returns the invoicePeriodeLengthInMonth
     *
     * @return int $invoicePeriodeLengthInMonth
     */
    public function getInvoicePeriodeLengthInMonth()
    {
        return $this->invoicePeriodeLengthInMonth;
    }

    /**
     * Sets the invoicePeriodeLengthInMonth
     *
     * @param int $invoicePeriodeLengthInMonth
     * @return void
     */
    public function setInvoicePeriodeLengthInMonth($invoicePeriodeLengthInMonth)
    {
        $this->invoicePeriodeLengthInMonth = $invoicePeriodeLengthInMonth;
    }


    /**
     * Returns the invoiceText
     *
     * @return string $invoiceText
     */
    public function getInvoiceText()
    {
        return $this->invoiceText;
    }

    /**
     * Sets the invoiceText
     *
     * @param string $invoiceText
     * @return void
     */
    public function setInvoiceText($invoiceText)
    {
        $this->invoiceText = $invoiceText;
    }

    /**
     * Get the value of useOwnAddressNumber
     *
     * @return string
     */
    public function getUseOwnAddressNumber(): bool
    {
        return $this->useOwnAddressNumber;
    }

    /**
     * Set the value of useOwnAddressNumber
     *
     * @param string $useOwnAddressNumber
     *
     * @return self
     */
    public function setUseOwnAddressNumber(bool $useOwnAddressNumber)
    {
        $this->useOwnAddressNumber = $useOwnAddressNumber;

        return $this;
    }
    
    /**
     * Returns the exportConfig
     *
     * @return string $exportConfig
     */
    public function getExportConfig()
    {
        return $this->exportConfig;
    }
    /**
     * Returns the exportConfig
     *
     * @return object $exportConfig
     */
    public function getExportConfigObject()
    {
        return json_decode($this->exportConfig);
    }

    /**
     * Sets the exportConfig
     *
     * @param string $exportConfig
     * @return void
     */
    public function setExportConfig($exportConfig)
    {
        $this->exportConfig = $exportConfig;
    }

    /**
     * Returns the logo
     */
    public function getLogo(): ?FileReference
    {
        return $this->logo;
    }

    /**
     * Sets the logo
     */
    public function setLogo(?FileReference $logo = null): void
    {
        $this->logo = $logo;
    }



    /**
     * Returns the logoWidth
     *
     * @return ?int $logoWidth
     */
    public function getLogoWidth()
    {
        return $this->logoWidth ? : null;
    }

    /**
     * Sets the logoWidth
     *
     * @param int $logoWidth
     * @return void
     */
    public function setLogoWidth(int $logoWidth = null)
    {
        $this->logoWidth = $logoWidth;
    }


    /**
     * Returns the logoMail
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $logoMail
     */
    public function getLogoMail()
    {
        return $this->logoMail;
    }

    /**
     * Sets the logoMail
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $logoMail
     * @return void
     */
    public function setLogoMail(\TYPO3\CMS\Extbase\Domain\Model\FileReference $logoMail=null)
    {
        $this->logoMail = $logoMail;
    }

    /**
     * Returns the logoMailWidth
     *
     * @return ?int $logoMailWidth
     */
    public function getLogoMailWidth()
    {
        return $this->logoMailWidth ? : null;
    }

    /**
     * Sets the logoMailWidth
     *
     * @param int $logoMailWidth
     * @return void
     */
    public function setLogoMailWidth(int $logoMailWidth = null)
    {
        $this->logoMailWidth = $logoMailWidth;
    }


    /**
     * Returns the street
     *
     * @return string $street
     */
    public function getStreet()
    {
        return $this->street;
    }

    /**
     * Sets the street
     *
     * @param string $street
     * @return void
     */
    public function setStreet($street)
    {
        $this->street = $street;
    }

    /**
     * Returns the streetNumber
     *
     * @return string $streetNumber
     */
    public function getStreetNumber()
    {
        return $this->streetNumber;
    }

    /**
     * Sets the streetNumber
     *
     * @param string $streetNumber
     * @return void
     */
    public function setStreetNumber($streetNumber)
    {
        $this->streetNumber = $streetNumber;
    }
    public function getAddress()
    {
        return $this->street . ($this->streetNumber ? ' ' . $this->streetNumber : '');
    }

    /**
     * Returns the address2
     *
     * @return string $address2
     */
    public function getAddress2()
    {
        return $this->address2;
    }

    /**
     * Sets the address2
     *
     * @param string $address2
     * @return void
     */
    public function setAddress2($address2)
    {
        $this->address2 = $address2;
    }

    /**
     * Returns the address3
     *
     * @return string $address3
     */
    public function getAddress3()
    {
        return $this->address3;
    }

    /**
     * Sets the address3
     *
     * @param string $address3
     * @return void
     */
    public function setAddress3($address3)
    {
        $this->address3 = $address3;
    }

    /**
     * Returns the countryIsoCode
     *
     * @return string $countryIsoCode
     */
    public function getCountryIsoCode()
    {
        return $this->countryIsoCode;
    }
    public function getCountryPostalCode()
    {
        return $this->countryIsoCode == 'LI' ? 'FL' : $this->countryIsoCode;
    }

    /**
     * Sets the countryIsoCode
     *
     * @param string $countryIsoCode
     * @return void
     */
    public function setCountryIsoCode($countryIsoCode)
    {
        $this->countryIsoCode = $countryIsoCode;
    }

    /**
     * Returns the zip
     *
     * @return string $zip
     */
    public function getZip()
    {
        return $this->zip;
    }

    /**
     * Sets the zip
     *
     * @param string $zip
     * @return void
     */
    public function setZip($zip)
    {
        $this->zip = $zip;
    }

    /**
     * Returns the city
     *
     * @return string $city
     */
    public function getCity()
    {
        return $this->city;
    }

    /**
     * Sets the city
     *
     * @param string $city
     * @return void
     */
    public function setCity($city)
    {
        $this->city = $city;
    }

    /**
     * Returns the cityDe
     *
     * @return string $cityDe
     */
    public function getCityDe()
    {
        return $this->cityDe;
    }

    /**
     * Sets the cityDe
     *
     * @param string $cityDe
     * @return void
     */
    public function setCityDe($cityDe)
    {
        $this->cityDe = $cityDe;
    }

    /**
     * Returns the cityFr
     *
     * @return string $cityFr
     */
    public function getCityFr()
    {
        return $this->cityFr;
    }

    /**
     * Sets the cityFr
     *
     * @param string $cityFr
     * @return void
     */
    public function setCityFr($cityFr)
    {
        $this->cityFr = $cityFr;
    }

    /**
     * Returns the cityIt
     *
     * @return string $cityIt
     */
    public function getCityIt()
    {
        return $this->cityIt;
    }

    /**
     * Sets the cityIt
     *
     * @param string $cityIt
     * @return void
     */
    public function setCityIt($cityIt)
    {
        $this->cityIt = $cityIt;
    }
    /**
     * Returns the cityEn
     *
     * @return string $cityEn
     */
    public function getCityEn()
    {
        return $this->cityEn;
    }

    /**
     * Sets the cityEn
     *
     * @param string $cityEn
     * @return void
     */
    public function setCityEn($cityEn)
    {
        $this->cityEn = $cityEn;
    }

    public function getCityLocale()
    {
        switch ($GLOBALS['TSFE']->sys_language_uid) {
       case 1:
           return $this->cityEn ? : $this->city;
       case 2:
           return $this->cityFr ? : $this->city;
       case 3:
           return $this->cityIt ? : $this->city;
       default:
           return $this->cityDe ? : $this->city;
       }
    }

    /**
     * Returns the phone
     *
     * @return string $phone
     */
    public function getPhone()
    {
        return $this->phone;
    }

    /**
     * Sets the phone
     *
     * @param string $phone
     * @return void
     */
    public function setPhone($phone)
    {
        $this->phone = $phone;
    }

    /**
     * Returns the aclOwner
     *
     * @return integer $aclOwner
     */
    public function getAclOwner()
    {
        return  $this->aclOwner;
    }

    /**
     * Sets the aclOwner
     *
     * @param integer $aclOwner
     * @return void
     */
    public function setAclOwner($aclOwner) {
        $this->aclOwner = $aclOwner;
    }

    /**
     * Sets the aclWriteGroups. Keep in mind that the property is called "aclWriteGroups"
     * although it can hold several aclWriteGroups.
     *
     * @param ObjectStorage<FrontendUserGroup> $aclWriteGroups
     */
    public function setAclWriteGroups(ObjectStorage $aclWriteGroups)
    {
        $this->aclWriteGroups = $aclWriteGroups;
    }

    /**
     * Adds a aclWriteGroups to the frontend user
     *
     * @param FrontendUserGroup $aclWriteGroups
     */
    public function addAclWriteGroups(FrontendUserGroup $aclWriteGroups)
    {
        $this->aclWriteGroups->attach($aclWriteGroups);
    }


    /**
     * Adds a FrontendUserGroup to aclWriteGroups (if not already exists)
     *
     * @param FrontendUserGroup $aclWriteGroups
     */
    public function addAclWriteGroupsIfNotContains(FrontendUserGroup $aclWriteGroups)
    {
        if (!$this->aclWriteGroups->contains($aclWriteGroups)) {
            $this->aclWriteGroups->attach($aclWriteGroups);
        }
    }

    /**
     * Removes a aclWriteGroups from the frontend user
     *
     * @param FrontendUserGroup $aclWriteGroups
     */
    public function removeAclWriteGroups(FrontendUserGroup $aclWriteGroups)
    {
        $this->aclWriteGroups->detach($aclWriteGroups);
    }

    /**
     * Returns the aclWriteGroups
     *
     * @return ObjectStorage<FrontendUserGroup> An object storage containing the aclWriteGroups
     */
    public function getAclWriteGroups()
    {
        return $this->aclWriteGroups;
    }

    /**
     * Sets the aclReadGroups. Keep in mind that the property is called "aclReadGroups"
     * although it can hold several aclReadGroups.
     *
     * @param ObjectStorage<FrontendUserGroup> $aclReadGroups
     */
    public function setAclReadGroups(ObjectStorage $aclReadGroups)
    {
        $this->aclReadGroups = $aclReadGroups;
    }

    /**
     * Adds a aclReadGroups to the frontend user
     *
     * @param FrontendUserGroup $aclReadGroups
     */
    public function addAclReadGroups(FrontendUserGroup $aclReadGroups)
    {
        $this->aclReadGroups->attach($aclReadGroups);
    }

    /**
     * Adds a FrontendUserGroup to aclWriteGroups (if not already exists)
     *
     * @param FrontendUserGroup $aclWriteGroups
     */
    public function addAclReadGroupsIfNotContains(FrontendUserGroup $aclReadGroups)
    {
        if (!$this->aclReadGroups->contains($aclReadGroups)) {
            $this->aclReadGroups->attach($aclReadGroups);
        }
    }

    /**
     * Removes a aclReadGroups from the frontend user
     *
     * @param FrontendUserGroup $aclReadGroups
     */
    public function removeAclReadGroups(FrontendUserGroup $aclReadGroups)
    {
        $this->aclReadGroups->detach($aclReadGroups);
    }

    /**
     * Returns the aclReadGroups
     *
     * @return ObjectStorage<FrontendUserGroup> An object storage containing the aclReadGroups
     */
    public function getAclReadGroups()
    {
        return $this->aclReadGroups;
    }

    public function aclHasViewPermission(array $groups)
    {
        $aclGroupArray=explode(',', $this->aclReadGroups);
        foreach ($groups as $groupId) {
            if (in_array($groupId, $aclGroupArray)) {
                return true;
            }
        }
        return false;
    }








    /**
     * Sets the languages. Keep in mind that the property is called "languages"
     * although it can hold several languages.
     *
     * @param ObjectStorage<Language> $languages
     */
    public function setLanguages(ObjectStorage $languages)
    {
        /*
        foreach ($languages as $language) {
            var_dump($language);
        }die('a');
        */
        $this->languages = $languages;
    }

    /**
     * Adds a languages to the frontend user
     *
     * @param Language $languages
     */
    public function addLanguages(Language $languages)
    {
        $this->languages->attach($languages);
    }

    /**
     * Adds a Language to aclWriteGroups (if not already exists)
     *
     * @param Language $aclWriteGroups
     */
    public function addLanguagesIfNotContains(Language $languages)
    {
        if (!$this->languages->contains($languages)) {
            $this->languages->attach($languages);
        }
    }

    /**
     * Removes a languages from the frontend user
     *
     * @param Language $languages
     */
    public function removeLanguages(Language $languages)
    {
        $this->languages->detach($languages);
    }

    /**
     * Returns the languages
     *
     * @return ObjectStorage<Language> An object storage containing the languages
     */
    public function getLanguages()
    {
        return $this->languages;
    }









    /**
     * Returns the editJs
     *
     * @return string $editJs
     */
    public function getEditJs()
    {
        return $this->editJs;
    }

    /**
     * Sets the editJs
     *
     * @param string $editJs
     * @return void
     */
    public function setEditJs($editJs)
    {
        $this->editJs = $editJs;
    }

    /**
     * Returns the editCss
     *
     * @return string $editCss
     */
    public function getEditCss()
    {
        return $this->editCss;
    }

    /**
     * Sets the editCss
     *
     * @param string $editCss
     * @return void
     */
    public function setEditCss($editCss)
    {
        $this->editCss = $editCss;
    }

    /**
     * Returns the templateName
     *
     * @return string $templateName
     */
    public function getTemplateName()
    {
        return $this->templateName;
    }

    /**
     * Sets the templateName
     *
     * @param string $templateName
     * @return void
     */
    public function setTemplateName($templateName)
    {
        $this->templateName = $templateName;
    }
    /**
     * Returns the titleExtraTextfield1
     *
     * @return string $titleExtraTextfield1
     */
    public function getTitleExtraTextfield1()
    {
        return $this->titleExtraTextfield1;
    }

    /**
     * Sets the titleExtraTextfield1
     *
     * @param string $titleExtraTextfield1
     * @return void
     */
    public function setTitleExtraTextfield1($titleExtraTextfield1)
    {
        $this->titleExtraTextfield1 = $titleExtraTextfield1;
    }

    /**
     * Returns the titleExtraTextfield2
     *
     * @return string $titleExtraTextfield2
     */
    public function getTitleExtraTextfield2()
    {
        return $this->titleExtraTextfield2;
    }

    /**
     * Sets the titleExtraTextfield2
     *
     * @param string $titleExtraTextfield2
     * @return void
     */
    public function setTitleExtraTextfield2($titleExtraTextfield2)
    {
        $this->titleExtraTextfield2 = $titleExtraTextfield2;
    }

    /**
     * Returns the titleExtraTextfield3
     *
     * @return string $titleExtraTextfield3
     */
    public function getTitleExtraTextfield3()
    {
        return $this->titleExtraTextfield3;
    }

    /**
     * Sets the titleExtraTextfield3
     *
     * @param string $titleExtraTextfield3
     * @return void
     */
    public function setTitleExtraTextfield3($titleExtraTextfield3)
    {
        $this->titleExtraTextfield3 = $titleExtraTextfield3;
    }

    /**
     * Returns the titleExtraTextarea1
     *
     * @return string $titleExtraTextarea1
     */
    public function getTitleExtraTextarea1()
    {
        return $this->titleExtraTextarea1;
    }

    /**
     * Sets the titleExtraTextarea1
     *
     * @param string $titleExtraTextarea1
     * @return void
     */
    public function setTitleExtraTextarea1($titleExtraTextarea1)
    {
        $this->titleExtraTextarea1 = $titleExtraTextarea1;
    }

    /**
     * Returns the titleExtraTextarea2
     *
     * @return string $titleExtraTextarea2
     */
    public function getTitleExtraTextarea2()
    {
        return $this->titleExtraTextarea2;
    }

    /**
     * Sets the titleExtraTextarea2
     *
     * @param string $titleExtraTextarea2
     * @return void
     */
    public function setTitleExtraTextarea2($titleExtraTextarea2)
    {
        $this->titleExtraTextarea2 = $titleExtraTextarea2;
    }

    /**
     * Returns the titleExtraTextarea3
     *
     * @return string $titleExtraTextarea3
     */
    public function getTitleExtraTextarea3()
    {
        return $this->titleExtraTextarea3;
    }

    /**
     * Sets the titleExtraTextarea3
     *
     * @param string $titleExtraTextarea3
     * @return void
     */
    public function setTitleExtraTextarea3($titleExtraTextarea3)
    {
        $this->titleExtraTextarea3 = $titleExtraTextarea3;
    }

    /**
     * Returns the titleExternalId
     *
     * @return string $titleExternalId
     */
    public function getTitleExternalId()
    {
        return $this->titleExternalId;
    }

    /**
     * Sets the titleExternalId
     *
     * @param string $titleExternalId
     * @return void
     */
    public function setTitleExternalId($titleExternalId)
    {
        $this->titleExternalId = $titleExternalId;
    }

    /**
     * Returns the titleTags
     *
     * @return string $titleTags
     */
    public function getTitleTags()
    {
        return $this->titleTags;
    }

    /**
     * Sets the titleTags
     *
     * @param string $titleTags
     * @return void
     */
    public function setTitleTags($titleTags)
    {
        $this->titleTags = $titleTags;
    }

    /**
     * Returns the titleCommissions
     *
     * @return string $titleCommissions
     */
    public function getTitleCommissions()
    {
        return $this->titleCommissions;
    }

    /**
     * Sets the titleCommissions
     *
     * @param string $titleCommissions
     * @return void
     */
    public function setTitleCommissions($titleCommissions)
    {
        $this->titleCommissions = $titleCommissions;
    }

    /**
     * Returns the titleCategories
     *
     * @return string $titleCategories
     */
    public function getTitleCategories()
    {
        return $this->titleCategories;
    }

    /**
     * Sets the titleCategories
     *
     * @param string $titleCategories
     * @return void
     */
    public function setTitleCategories($titleCategories)
    {
        $this->titleCategories = $titleCategories;
    }

    /**
     * Returns the veteranenAlter
     *
     * @return int $veteranenAlter
     */
    public function getVeteranenAlter()
    {
        return $this->veteranenAlter;
    }

    /**
     * Sets the veteranenAlter
     *
     * @param int $veteranenAlter
     * @return void
     */
    public function setVeteranenAlter($veteranenAlter)
    {
        $this->veteranenAlter = $veteranenAlter;
    }

    /**
     * Returns the alterskategorieMode
     *
     * @return int $alterskategorieMode
     */
    public function getAlterskategorieMode()
    {
        return $this->alterskategorieMode;
    }

    /**
     * Sets the alterskategorieMode
     *
     * @param int $alterskategorieMode
     * @return void
     */
    public function setAlterskategorieMode($alterskategorieMode)
    {
        $this->alterskategorieMode = $alterskategorieMode;
    }

    /**
     * Returns the journalMode
     *
     * @return int $journalMode
     */
    public function getJournalMode()
    {
        return $this->journalMode;
    }

    /**
     * Sets the journalMode
     *
     * @param int $journalMode
     * @return void
     */
    public function setJournalMode($journalMode)
    {
        $this->journalMode = $journalMode;
    }

    /**
     * Returns the invoiceMode
     *
     * @return int $invoiceMode
     */
    public function getInvoiceMode()
    {
        return $this->invoiceMode;
    }

    /**
     * Sets the invoiceMode
     *
     * @param int $invoiceMode
     * @return void
     */
    public function setInvoiceMode($invoiceMode)
    {
        $this->invoiceMode = $invoiceMode;
    }






    /**
     * Returns the personUidAttribute
     *
     * @return string $personUidAttribute
     */
    public function getPersonUidAttribute()
    {
        return $this->personUidAttribute;
    }

    /**
     * Sets the personUidAttribute
     *
     * @param string $personUidAttribute
     * @return void
     */
    public function setPersonUidAttribute($personUidAttribute)
    {
        $this->personUidAttribute = $personUidAttribute;
    }




    /**
     * Returns the titleOrganisationId
     *
     * @return string $titleOrganisationId
     */
    public function getTitleOrganisationId()
    {
        return $this->titleOrganisationId;
    }

    /**
     * Sets the titleOrganisationId
     *
     * @param string $titleOrganisationId
     * @return void
     */
    public function setTitleOrganisationId($titleOrganisationId)
    {
        $this->titleOrganisationId = $titleOrganisationId;
    }

    /**
     * Returns the titleJournalIsFree1
     *
     * @return string $titleJournalIsFree1
     */
    public function getTitleJournalIsFree1()
    {
        return $this->titleJournalIsFree1;
    }

    /**
     * Sets the titleJournalIsFree1
     *
     * @param string $titleJournalIsFree1
     * @return void
     */
    public function setTitleJournalIsFree1($titleJournalIsFree1)
    {
        $this->titleJournalIsFree1 = $titleJournalIsFree1;
    }

    /**
     * Returns the titleJournalIsFree2
     *
     * @return string $titleJournalIsFree2
     */
    public function getTitleJournalIsFree2()
    {
        return $this->titleJournalIsFree2;
    }

    /**
     * Sets the titleJournalIsFree2
     *
     * @param string $titleJournalIsFree2
     * @return void
     */
    public function setTitleJournalIsFree2($titleJournalIsFree2)
    {
        $this->titleJournalIsFree2 = $titleJournalIsFree2;
    }


    /**
     * Returns the invoiceBaseText
     *
     * @return string $invoiceBaseText
     */
    public function getInvoiceBaseText()
    {
        return $this->invoiceBaseText;
    }

    /**
     * Sets the invoiceBaseText
     *
     * @param string $invoiceBaseText
     * @return void
     */
    public function setInvoiceBaseText($invoiceBaseText)
    {
        $this->invoiceBaseText = $invoiceBaseText;
    }



    /**
     * Returns the extraBetragTitle
     *
     * @return string $extraBetragTitle
     */
    public function getExtraBetragTitle()
    {
        return $this->extraBetragTitle;
    }

    /**
     * Sets the extraBetragTitle
     *
     * @param string $extraBetragTitle
     * @return void
     */
    public function setExtraBetragTitle($extraBetragTitle)
    {
        $this->extraBetragTitle = $extraBetragTitle;
    }

    /**
     * Returns the extraBetragValue
     *
     * @return float $extraBetragValue
     */
    public function getExtraBetragValue()
    {
        return $this->extraBetragValue;
    }

    /**
     * Sets the extraBetragValue
     *
     * @param float $extraBetragValue
     * @return void
     */
    public function setExtraBetragValue($extraBetragValue)
    {
        $this->extraBetragValue = $extraBetragValue;
    }


    /**
     * Returns the testDates
     *
     * @return boolean $testDates
     */
    public function getTestDates()
    {
        return $this->testDates;
    }

    /**
     * Sets the testDates
     *
     * @param boolean $testDates
     * @return void
     */
    public function setTestDates($testDates)
    {
        $this->testDates = $testDates;
    }

    /**
     * Returns the boolean state of testDates
     *
     * @return boolean
     */
    public function isTestDates()
    {
        return $this->testDates;
    }


    /**
     * Returns the showInvoiceAddress
     *
     * @return boolean $showInvoiceAddress
     */
    public function getShowInvoiceAddress()
    {
        return $this->showInvoiceAddress;
    }

    /**
     * Sets the showInvoiceAddress
     *
     * @param boolean $showInvoiceAddress
     * @return void
     */
    public function setShowInvoiceAddress($showInvoiceAddress)
    {
        $this->showInvoiceAddress = $showInvoiceAddress;
    }

    /**
     * Returns the boolean state of showInvoiceAddress
     *
     * @return boolean
     */
    public function isShowInvoiceAddress()
    {
        return $this->showInvoiceAddress;
    }

    /**
     * Adds a Export
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Export $export
     * @return void
     */
    public function addExport(\Internetgalerie\IgsCrm\Domain\Model\Export $export)
    {
        $this->exports->attach($export);
    }

    /**
     * Removes a Export
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Export $exportToRemove The Export to be removed
     * @return void
     */
    public function removeExport(\Internetgalerie\IgsCrm\Domain\Model\Export $exportToRemove)
    {
        $this->exports->detach($exportToRemove);
    }

    /**
     * Returns the exports
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Export> $exports
     */
    public function getExports()
    {
        return $this->exports;
    }

    /**
     * Sets the exports
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Export> $exports
     * @return void
     */
    public function setExports(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $exports)
    {
        $this->exports = $exports;
    }


    public function getCategories()
    {
        if ($this->categoryRepository===null) {
            $this->categoryRepository = GeneralUtility::makeInstance(\Internetgalerie\IgsCrm\Domain\Repository\CategoryRepository::class);
        }
        return $this->categoryRepository->findByVerband($this);
    }
    public function getTags()
    {
        if ($this->tagRepository===null) {
            $this->tagRepository = GeneralUtility::makeInstance(\Internetgalerie\IgsCrm\Domain\Repository\TagRepository::class);
        }
        return $this->tagRepository->findByVerband($this);
    }

    public function getJournals()
    {
        if ($this->zeitschriftRepository === null) {
            $this->zeitschriftRepository = GeneralUtility::makeInstance(ZeitschriftRepository::class);
        }
        return $this->zeitschriftRepository->findByVerband($this);
    }

    public function getTagsverband()
    {
        if ($this->tagverbandRepository === null) {
            $this->tagverbandRepository = GeneralUtility::makeInstance(\Internetgalerie\IgsCrm\Domain\Repository\TagverbandRepository::class);
        }
        return $this->tagverbandRepository->findByVerband($this);
    }

    public function getMitgliedschaften()
    {
        if ($this->mitgliedschaftRepository===null) {
            $this->mitgliedschaftRepository = GeneralUtility::makeInstance(\Internetgalerie\IgsCrm\Domain\Repository\MitgliedschaftRepository::class);
        }
        return $this->mitgliedschaftRepository->findByVerband($this);
    }

    /**
     * Get the value of invoiceCss
     *
     * @return string
     */
    public function getInvoiceCss()
    {
        return $this->invoiceCss;
    }

    /**
     * Set the value of invoiceCss
     *
     * @param string $invoiceCss
     *
     * @return self
     */
    public function setInvoiceCss($invoiceCss)
    {
        $this->invoiceCss = $invoiceCss;

        return $this;
    }

    /**
     * Returns the taxRate
     *
     * @return float $taxRate
     */
    public function getTaxRate()
    {
        return $this->taxRate;
    }
    public function getMwst()
    {
        return $this->taxRate;
    }

    /**
     * Sets the taxRate
     *
     * @param  float $taxRate
     * @return void
     */
    public function setTaxRate($taxRate)
    {
        $this->taxRate = $taxRate;
    }

    /**
     * Returns the taxBehavior
     *
     * @return int $taxBehavior
     */
    public function getTaxBehavior()
    {
        return $this->taxBehavior;
    }
    public function getTaxBehaviorIsInkl()
    {
        return $this->taxBehavior === Invoice::TAXBEHAVIOR_INKL;
    }
    public function getTaxBehaviorIsExkl()
    {
        return $this->taxBehavior === Invoice::TAXBEHAVIOR_EXKL;
    }
    public function getTaxBehaviorIsUnspecified()
    {
        return $this->taxBehavior === null;
    }

    /**
     * Sets the taxBehavior
     *
     * @param  int $taxBehavior
     * @return void
     */
    public function setTaxBehavior(int $taxBehavior = null)
    {
        $this->taxBehavior = $taxBehavior;
    }
    /**
     * Get the value of usePp
     *
     * @return string
     */
    public function getUsePp()
    {
        return $this->usePp;
    }

    /**
     * Set the value of usePp
     *
     * @param string $usePp
     *
     * @return self
     */
    public function setUsePp($usePp)
    {
        $this->usePp = $usePp;

        return $this;
    }

    public function getMitgliedschaftenExclude()
    {
        if ($this->mitgliedschaftenExclude === null) {
            $mitgliedschaftRepository = GeneralUtility::makeInstance(\Internetgalerie\IgsCrm\Domain\Repository\MitgliedschaftRepository::class);
            $this->mitgliedschaftenExclude = $mitgliedschaftRepository->findByVerbandExcludedStatistik($this->getUid(), 1);
        }
        return $this->mitgliedschaftenExclude;
    }

    public function getCards()
    {
        if ($this->cards === null) {
            $cardRepository = GeneralUtility::makeInstance(\Internetgalerie\IgsCrm\Domain\Repository\CardRepository::class);
            $this->cards = $cardRepository->findByVerband($this->getUid());
        }
        return $this->cards;
    }
    
    public function getTemplateLetters()
    {
        if ($this->templateLetters === null) {
            $templateLetterRepository = GeneralUtility::makeInstance(TemplateLetterRepository::class);
            $this->templateLetters = $templateLetterRepository->findByTenantId($this->getUid());
        }
        return $this->templateLetters;
    }

    public function getTemplateLetterFavorites()
    {
        $templateLetterRepository = GeneralUtility::makeInstance(TemplateLetterRepository::class);
        return  $templateLetterRepository->findFavoritesByVerbandType($this->getUid(), 'Letter', null);
    }


    //public function getTemplateLetter(string $templateType, string $objectType, \Internetgalerie\IgsCrm\Domain\Model\Language $crmLanguage = null, Invoice $invoice = null)
    /*
    public function getTemplateLetter(string $templateType, Contact $contact, Invoice $invoice = null)
    {
        $objectType = $contact->getType();
        $crmLanguage = $contact->getLanguageWithFallback(['de', 'fr']);
        if ($invoice !== null) {
            $mitgliedschaftUid = $invoice->getMitgliedschaft();
        } else {
            $mitgliedschaftUid = 0;
        }
        CrmTranslate::setVerband($this->uid);
        if ($crmLanguage instanceof \Internetgalerie\IgsCrm\Domain\Model\Language) {
            $crmLanguageUid = $crmLanguage->getUid();
            if (isset($this->templateLetter[$templateType][$objectType][$crmLanguageUid][$mitgliedschaftUid])) {
                CrmTranslate::setCrmLanguage($crmLanguageUid);
                return $this->templateLetter[$templateType][$objectType][$crmLanguageUid][$mitgliedschaftUid];
            }
        } else {
            $crmLanguageUid = 0;
        }
        CrmTranslate::setCrmLanguage($crmLanguageUid);

        if (!isset($this->templateLetter[$templateType])) {
            $this->templateLetter[$templateType] = [];
        }
        if (!isset($this->templateLetter[$templateType][$objectType])) {
            $this->templateLetter[$templateType][$objectType] = [];
        }
        if (!isset($this->templateLetter[$templateType][$objectType][$crmLanguageUid])) {
            $this->templateLetter[$templateType][$objectType][$crmLanguageUid] = [];
        }
        $templateLetterRepository = GeneralUtility::makeInstance(TemplateLetterRepository::class);

        $template = null;

        
        if ($mitgliedschaftUid) {
            $template = $templateLetterRepository->findOneByVerbandTypeLanguageMainTag($this->uid, $templateType, $invoice, $objectType, $crmLanguageUid, $mitgliedschaftUid);
            if ($template === null) {
                $template = $templateLetterRepository->findOneByVerbandTypeLanguageMainTag($this->uid, $templateType, $invoice, $objectType, null, $mitgliedschaftUid);
            }
        }

        if ($template === null) {
            if (isset($this->templateLetter[$templateType][$objectType][$crmLanguageUid]['0'])) {
                $template = $this->templateLetter[$templateType][$objectType][$crmLanguageUid]['0'];
            } else {
                $template = $templateLetterRepository->findOneByVerbandTypeLanguage($this->uid, $templateType, $invoice, $objectType, $crmLanguageUid);
                if ($template === null) {
                    $template = $templateLetterRepository->findOneByVerbandTypeLanguage($this->uid, $templateType, $invoice, $objectType, null);
                }
            }
        }
        if ($template === null) {
            echo '<h1>Error: No template found</h1>';
            echo 'Verband: ' . $this->getName() . ' ('.$this->uid.')<br />';
            echo 'Template Type: ' . $templateType .'<br />';
            if ($invoice) {
                $invoiceType = $invoice->getType();
                $invoiceStatus = $invoice->getStatus() ? $invoice->getStatus()->getTitle() : '';
                echo 'Invoice Type: ' . $invoiceType .'<br />';
                echo 'Invoice Status: ' . $invoiceStatus .'<br />';
            }
            echo 'Object Type: ' . $objectType .'<br />';
            echo 'CRM Sprache: ' . $crmLanguageUid .'<br />';
            echo 'Mitgliedschaft: ' . ($mitgliedschaftUid ?? '[[ohne]]') .'<br />';
            exit(0);
        }
        $this->templateLetter[$templateType][$objectType][$crmLanguageUid][$mitgliedschaftUid] = $template;
        return $this->templateLetter[$templateType][$objectType][$crmLanguageUid][$mitgliedschaftUid];

    }
    */
    public function getTranslates()
    {
        if ($this->translates === null) {
            $translateRepository = GeneralUtility::makeInstance(TranslateRepository::class);
            $this->translates = $translateRepository->findByTenantId($this->getUid());
            //var_dump($this->translates->count(), $this->getUid());exit(0);
        }
        return $this->translates;

    }
    public function getReferenceFormatterClass(): string
    {
        return DefaultReferenceFormatter::class;
    }
    
    public function getLanguageCode()
    {
        return 'de';
    }
    public function setAddon(string $addon): void
    {
        $this->addon = $addon;
    }
    
    public function getAddon()
    {
        return $this->addon;
    }
    public function getPoboxText()
    {
        return '';
    }
    public function getIsOrganisation()
    {
        return true;
    }

    /**
     * Get the value of senderLine
     *
     * @return string
     */
    public function getSenderLine()
    {
        return $this->senderLine;
    }

    /**
     * Set the value of senderLine
     *
     * @param string $senderLine
     *
     * @return self
     */
    public function setSenderLine($senderLine)
    {
        $this->senderLine = $senderLine;

        return $this;
    }

    public function __toString()
    {
        return $this->getName();
    }

}
