<?php
namespace Internetgalerie\IgsCrm\Domain\Model;

/**
 * Zeitschrift
 */
class Zeitschrift extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{
    /**
     * name
     *
     * @var string
     */
    protected $name = '';

    /**
     * betrag
     *
     * @var float
     */
    protected $betrag = 0.0;

    /**
     * verband
     *
     * @var \Internetgalerie\IgsCrm\Domain\Model\Verband
     */
    public $verband = null;

    /**
     * preise
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Preise>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    public $preise = null;

    /**
     * __construct
     */
    public function __construct()
    {
        //Do not remove the next line: It would break the functionality
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    protected function initStorageObjects()
    {
        $this->preise = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Returns the name
     *
     * @return string $name
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Sets the name
     *
     * @param string $name
     * @return void
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Returns the betrag
     *
     * @return float $betrag
     */
    public function getBetrag()
    {
        return $this->betrag;
    }


    public function getBetragWithAlter($alterInJahre)
    {
        if (intval($alterInJahre)<=0) {
            return $this->betrag;
        }
        foreach ($this->getPreise() as $preis) {
            if ($preis->getAlterInJahrVon()<=$alterInJahre && $preis->getAlterInJahrBis()>=$alterInJahre) {
                return $preis->getBetrag();
            }
        }
        return $this->betrag;
    }

    /**
     * Sets the betrag
     *
     * @param float $betrag
     * @return void
     */
    public function setBetrag($betrag)
    {
        $this->betrag = $betrag;
    }

    /**
     * Returns the verband
     *
     * @return \Internetgalerie\IgsCrm\Domain\Model\Verband $verband
     */
    public function getVerband()
    {
        return $this->verband;
    }

    /**
     * Sets the verband
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Verband $verband
     * @return void
     */
    public function setVerband(\Internetgalerie\IgsCrm\Domain\Model\Verband $verband)
    {
        $this->verband = $verband;
    }

    /**
     * Adds a Preise
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Preise $preise
     * @return void
     */
    public function addPreise(\Internetgalerie\IgsCrm\Domain\Model\Preise $preise)
    {
        $this->preise->attach($preise);
    }

    /**
     * Removes a Preise
     *
     * @param \Internetgalerie\IgsCrm\Domain\Model\Preise $preiseToRemove The Preise to be removed
     * @return void
     */
    public function removePreise(\Internetgalerie\IgsCrm\Domain\Model\Preise $preiseToRemove)
    {
        $this->preise->detach($preiseToRemove);
    }

    /**
     * Returns the preise
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Preise> $preise
     */
    public function getPreise()
    {
        return $this->preise;
    }

    /**
     * Sets the preise
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Internetgalerie\IgsCrm\Domain\Model\Preise> $preise
     * @return void
     */
    public function setPreise(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $preise)
    {
        $this->preise = $preise;
    }
    public function getNamePrice()
    {
        return $this->name . ' (CHF' . number_format($this->betrag, 2) . ')';
    }
    public function __toString()
    {
        return $this->getName();
    }
    public function getInvoicestext()
    {
        return $this->name;
    }
}
