<?php
namespace Internetgalerie\IgsCrm\Domain\Repository;

use Internetgalerie\IgsCrm\Domain\Model\Verband;
use Internetgalerie\IgsCrm\Utility\ConfUtility;
use Internetgalerie\IgsCrm\Utility\SecurityUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\Connection;

/**
 * abstrace crm repository for acls
 */
abstract class AbstractCrmRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
{

    /**
     * confUtility
     *
     * @var ConfUtility
     */
    protected $confUtility = null;


    /**
     * securityUtility
     *
     * @var SecurityUtility
     */
    protected $securityUtility = null;


    /**
     * use acl for queries
     *
     * @var bool
     */
    protected $useAcl = true;

    /**
     * security token
     *
     * @var string
     */
    protected $token = null;
    

    /**
     * Inject the ConfUtility (for checking access to objects)
     *
     * @param ConfUtility $confUtility
     */
    public function injectConfUtility(ConfUtility $confUtility)
    {
        $this->confUtility = $confUtility;
    }

    /**
     * Inject the SecurityUtility (for checking access to objects)
     *
     * @param SecurityUtility $securityUtility
     */
    public function injectSecurityUtility(SecurityUtility $securityUtility)
    {
        $this->securityUtility = $securityUtility;
    }

    public function getToken()
    {
        return $this->token;
    }
    public function setToken(bool $token)
    {
        $this->setUseAcl(false);
        return $this->token = $token;
    }
    
    public function getUseAcl()
    {
        return $this->useAcl;
    }
    public function setUseAcl(bool $useAcl)
    {
        return $this->useAcl = $useAcl;
    }
    
    public function getQueryConstraintsWithAcl($query)
    {
        $constraints = [];
        if (!$this->useAcl) {
            return $constraints;
        }
        if (!$this->securityUtility->isAdmin()) {
             $orConstraints = [
                $query->equals('aclOwner', $this->securityUtility->getFrontendUserId()),
             ];
            foreach ($this->securityUtility->getFrontendUserGroupIds() as $userGroupId) {
                // is inSet / "FIND_IN_SET(" . (int) $userGroupId . " ,acl_read_groups)"
                $orConstraints[] =  $query->contains('aclReadGroups', $userGroupId);
                /*
                  // alternatic
                  $orConstraints[] =  $query->like('aclReadGroups', $userGroupId .',%'); // am Anfang
                  $orConstraints[] =  $query->like('aclReadGroups', .'%,'$userGroupId.',%'); // in der Mitte
                  $orConstraints[] =  $query->like('aclReadGroups', '%,' . $userGroupId); // am schluss
                 */
            }
            //var_dump($orConstraints);exit(0);
            if (!empty($orConstraints)) {
                $constraints[] = $query->logicalOr(...$orConstraints);
            }
            
            //$constraints[] = $query->in('aclWriteGroups', $this->securityUtility->getFrontendUserGroupIds());
        }
        return $constraints;
    }
    public function getQueryBuilderConstraintsWithAcl($queryBuilder)
    {
        $constraints = [];
        if (!$this->securityUtility->isAdmin()) {
            $frontendUserGroupIds = $this->securityUtility->getFrontendUserGroupIds();

            $orConstraints = [
                $queryBuilder->expr()->eq('acl_owner', $this->securityUtility->getFrontendUserId()),
            ];
            foreach ($this->securityUtility->getFrontendUserGroupIds() as $userGroupId) {
                //$orConstraints[] =  "FIND_IN_SET(" . (int) $userGroupId . " ,acl_write_groups)";
                $orConstraints[] =  $queryBuilder->expr()->inSet('acl_read_groups', $userGroupId);
            }
            //var_dump($orConstraints);exit(0);
            if (!empty($orConstraints)) {
                $constraints[] = $queryBuilder->expr()->orX(...$orConstraints);
            }


            //$constraints[] = $queryBuilder->expr()->in('acl_write_groups', $queryBuilder->createNamedParameter($frontendUserGroupIds, Connection::PARAM_INT_ARRAY));
        }
        return $constraints;
    }
}